@@ -11,6 +11,7 @@ enum eCommandKind {
     kVar, kGlobal, kExport, 
     kAry, kAryAdd, kAryErase, 
     kHash, kHashAdd, kHashErase,  
+    kDef, kClass,
     kSubshell, kMSleep, kTrue, kFalse, kTest,
     kIndex, kRIndex, kLength, kLc, kUc,
     kChomp, kSubstr, kEval, kFg, kBg, 
@@ -36,7 +37,6 @@ enum eCommandKind {
     kRange,
     kPrintf,
     kPomch,
-    kDef,
     kPTee,
     kPCat,
     kReturn,
@@ -58,6 +58,8 @@ enum eCommandKind {
     kFAbs,
     kP,
     kTry,
+    kEnvSpace,
+    kQuote,
     kInnerCommand,
     kCommand 
 };
@@ -70,6 +72,10 @@ extern hash_obj* gFuncs;
     // 桼ؿ sFunction*äƤ
     // ϴؿ̾
 
+extern hash_obj* gClasses;
+    // 饹 sClass*äƤ
+    // ϥ饹̾
+
 extern vector_obj* gStackFrame;
     // ѿΥå
     // Ƥhash_obj*
@@ -131,7 +137,7 @@ extern vector_obj* gDirStack;
 extern enum eLineField gLineField;
     // saphireʸޥɤǻȤԥ
 
-extern vector_obj* gPSubTmpFiles;
+extern vector_obj* gTmpFiles;
     // ץִǻȤեǾä
     // ¸Ƥ
 
@@ -158,6 +164,9 @@ void null_fun();
 
 void get_quoted_linefield(char* str, string_obj* out);
 
+void bash_get_quoted_fname(char* fname, string_obj* quoted_fname);
+    // bashΥ
+
 typedef struct _sJob {
     string_obj* mName;          // ֤̾
     pid_t mPGroup;              // ץ롼ID
@@ -250,7 +259,15 @@ sBlock* sBlock_load(int fd);
 
 typedef struct
 {
-    int mKind; // 0:string 2: statments 4: statments2 5: block 6: process substituion
+    string_obj* mFName;
+    sStatments* mStatments;
+} sPSub;
+
+sPSub* sPSub_new(char* fname, sStatments* statments);
+
+typedef struct
+{
+    int mKind; // 0:string 2: statments 4: statments2 5: block 6: process substituion 7: here document 8:process substituion2
     void* mBody;
 } sArg; // mArgs
 
@@ -348,13 +365,16 @@ sFunction* sFunction_load(int fd);
 void sFunction_view(sFunction* self);
 
 typedef struct {
-    pid_t mPid;
-    string_obj* mTitle;
-    BOOL mWriterProcess;
-} sChildProgram;
+    string_obj* name;      // 饹̾
+    sStatments* statments; // ̿ɽ
+} sClass;
 
-sChildProgram* sChildProgram_new(pid_t pid, char* title, BOOL writer_process);
-void sChildProgram_delete(sChildProgram* self);
+sClass* sClass_new(char* name, sStatments* statments);
+sClass* sClass_new2(sClass* klass);
+  
+void sClass_save(sClass* self, int fd);
+sClass* sClass_load(int fd);
+void sClass_view(sClass* self);
 
 typedef struct {
     string_obj* mName;
@@ -366,16 +386,19 @@ void sInnerCommand_delete(sInnerCommand* self);
 
 void sigchld_block(int block);
 
+extern hash_obj* gObjects;
+// ֥ ϥ֥̾ ƤsObject*
+
 ////////////////////////////////////////////////////////////
 /// from saphire_commands.c
 ////////////////////////////////////////////////////////////
-int statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, int j, char* title, BOOL input, BOOL* return_, char* sname, int line);
+int statment_tree_internal_commands(sCommand* command, int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, int j, char* title, BOOL input, BOOL* return_, char* sname, int line, vector_obj* envs, sObject* object);
 BOOL vcat_expand_env(string_obj* pipeout, char* env, vector_obj* numbers,char* sname, int line);
 
 ////////////////////////////////////////////////////////////
 /// from saphire_parser.c
 ////////////////////////////////////////////////////////////
-BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* line);
+BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* line, sObject* object);
 BOOL parse(char* cmdline, char* sname, int* line, sStatments* statments);
 BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname, int* line);
 
@@ -402,8 +425,15 @@ extern vector_obj* gStackTraceFName;
 extern vector_obj* gStackTraceLineNum;
     // åե졼εϿ
 
-int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, vector_obj* parent_blocks, BOOL try_);
+int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, vector_obj* parent_blocks, BOOL try_, vector_obj* envs, sObject* object);
     // saphire_compile2ǥѥ뤷ѥ̤¹Ԥ롣
-    // verboseϼ¹Ԥ˼¹Ԥ륳ޥ̾ɽ뤫ɤ
+
+int saphire_shell4(char* command, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, sObject* object);
+    // ʸɤ߹Ǽ¹
+int saphire_shell5(string_obj* result, char* command, char* title, sRFd* pipein, sObject* object);
+    // ʸɤ߹Ǽ¹ԡޥɤνϤresult
+void saphire_set_global_var2(char* name, char* value, sObject* object);
+    // ֥Ȥ°Хѿꤹ
+
 
 #endif
