@@ -23,6 +23,8 @@ string_obj* gErrMsg;
     // エラーメッセージ
 vector_obj* gSaphireProgCompletions;
     // 補完で使うプログラムの名前 vector of string
+vector_obj* gSaphireExternalProgCompletions;
+    // 補完で使うプログラムの名前 vector of string
 BOOL gKitutukiExit;
     // exitコマンドが実行されたらTRUE
 
@@ -37,6 +39,7 @@ char* gStatmentKindStrs[kCommand+1] = {
     "var", "global", "export", 
     "ary", "ary_add", "ary_erase", 
     "hash", "hash_add", "hash_erase",
+    "def", "class",
     "subshell", "msleep", "true",  "false", "[",  
     "index", 
     "rindex", 
@@ -68,7 +71,6 @@ char* gStatmentKindStrs[kCommand+1] = {
     "range",
     "printf",
     "pomch",
-    "def",
     "ptee",
     "pcat",
     "return",
@@ -89,6 +91,8 @@ char* gStatmentKindStrs[kCommand+1] = {
     "fabs",
     "p",
     "try",
+    "envspace",
+    "quote",
 //    "parse",
     "inner_command",
     "command"
@@ -97,6 +101,11 @@ char* gStatmentKindStrs[kCommand+1] = {
 hash_obj* gFuncs;
     // ユーザー関数 sFunction*が入っている
     // キーは関数名
+
+hash_obj* gClasses;
+    // クラス sClass*が入っている
+    // キーはクラス名
+
 hash_obj* gGlobals;
     // グローバル変数 キーは変数名 内容はstring_obj*
 hash_obj* gArrays;
@@ -162,9 +171,9 @@ vector_obj* gDirStack;
     // popd, pushdで使われるディレクトリスタック
     // 内容は string_obj*
 
-vector_obj* gPSubTmpFiles;
-    // プロセス置換で使われる一時ファイルを行末で消すために
-    // 保存しておく配列
+vector_obj* gTmpFiles;
+    // プロセス置換、ヘアドキュメントで使われる一時ファイルを行末で
+    // 消すために保存しておく配列
 
 vector_obj* gStackTraceFName;
 vector_obj* gStackTraceLineNum;
@@ -173,26 +182,31 @@ vector_obj* gStackTraceLineNum;
 BOOL gSaphireBackground;
     // Saphireがバックグラウンドで動いているかどうか
 
+hash_obj* gObjects;
+    // オブジェクト キーはオブジェクト名 内容はsObject*
+
+sObject* sObject_new(char* name, sObject* parent) 
+{
+    sObject* self = MALLOC(sizeof(sObject));
+
+    self->mName = STRING_NEW(name);
+    self->mGlobals = HASH_NEW(15);
+    self->mHashs = HASH_NEW(15);
+    self->mArrays = HASH_NEW(15);
+    self->mObjects = HASH_NEW(15);
+    self->mClasses = HASH_NEW(15);
+
+    self->mParent = parent;
+
+    self->mMethods = HASH_NEW(15);
+
+    return self;
+}
+
 //////////////////////////////////////////////////////////
 // 内部関数
 //////////////////////////////////////////////////////////
 
-// 引数がラインフィールドをポイントしているか
-BOOL is_line_field(char* p)
-{
-    if(gLineField == kLF && *p == '\n') {
-        return TRUE;
-    }
-    else if(gLineField == kCR && *p == '\r') {
-        return TRUE;
-    }
-    else if(gLineField == kCRLF && *p == '\r' && *(p+1) == '\n') {
-        return TRUE;
-    }
-
-    return FALSE;
-}
-
 // 引数がラインフィールドをポイントしているか2
 BOOL is_line_field2(enum eLineField lf, char* p)
 {
@@ -231,13 +245,6 @@ BOOL string_chomp(string_obj* str)
     return FALSE;
 }
 
-// 改行コードが無かったらつける(\nに統一)
-void string_ponch(string_obj* str)
-{
-    string_chomp(str);
-    string_push_back2(str, '\n');
-}
-
 // 改行コードを付ける(改行コードそのまま)
 void string_pomch(string_obj* str, enum eLineField lf)
 {
@@ -308,6 +315,27 @@ void saphire_get_quoted_fname(char* fname, string_obj* quoted_fname)
         }
     }
 }
+
+// 文字列をクォートして返す
+void bash_get_quoted_fname(char* fname, string_obj* quoted_fname)
+{
+    char* p = fname;
+    while(*p) {
+        if(*p==';' || *p=='&' || *p=='(' || *p==')' || *p=='|'
+                || *p=='^' || *p =='<' || *p=='>' || *p=='?'
+                || *p=='*' || *p=='[' || *p==']' || *p=='$' || *p=='`' 
+                || *p=='"' || *p=='\''
+                || *p=='{' || *p=='}' || *p=='\t' || *p==' ' || *p == '~'
+                || *p=='#' || *p=='\\')
+        {
+            string_push_back2(quoted_fname, '\\');
+            string_push_back2(quoted_fname, *p++);
+        }
+        else {
+            string_push_back2(quoted_fname, *p++);
+        }
+    }
+}
 /////////////////////////////////////////////////////////
 // ローカル変数、グローバル変数に参照、代入
 /////////////////////////////////////////////////////////
@@ -342,6 +370,16 @@ void saphire_set_global_var(char* name, char* value)
     hash_put(gGlobals, name, STRING_NEW(value));
 }
 
+void saphire_set_global_var2(char* name, char* value, sObject* object)
+{
+    string_obj* global = hash_item(object->mGlobals, name);
+
+    if(global) {
+        hash_erase(object->mGlobals, name);
+    }
+    hash_put(object->mGlobals, name, STRING_NEW(value));
+}
+
 
 char* saphire_get_global_var(char* name)
 {
@@ -849,6 +887,9 @@ sRedirect* sRedirect_load(int fd)
     return self;
 }
 
+/////////////////////////////////////////////////////////
+// 引数
+/////////////////////////////////////////////////////////
 sArg* sArg_new(int kind, void* body)
 {
     sArg* self = MALLOC(sizeof(sArg));
@@ -884,6 +925,14 @@ sArg* sArg_new2(sArg* arg)
     case 6:
         self->mBody = sStatments_new2(arg->mBody);
         break;
+
+    case 7:
+        self->mBody = STRING_NEW(string_c_str(arg->mBody));
+        break;
+
+    case 8:
+        self->mBody = sStatments_new2(arg->mBody);
+        break;
     }
 
     return self;
@@ -929,6 +978,26 @@ void sArg_save(sArg* self, int fd)
         case 6:
             sStatments_save(self->mBody, fd);
             break;
+
+        case 7: {
+            string_obj* str = self->mBody;
+            int n = string_size(str);
+
+            if(write(fd, &n, sizeof(int)) < 0) {
+                perror("write");
+                exit(1);
+            }
+
+            if(write(fd, string_c_str(str), n) < 0) {
+                perror("write");
+                exit(1);
+            }
+            }
+            break;
+
+        case 8:
+            sStatments_save(self->mBody, fd);
+            break;
     }
 }
 
@@ -976,14 +1045,45 @@ sArg* sArg_load(int fd)
     case 6:
         self->mBody = sStatments_load(fd);
         break;
+
+    case 7: {
+        int m;
+        if(read(fd, &m, sizeof(int)) < 0) {
+            perror("read");
+            exit(1);
+        }
+        char* buf = MALLOC(sizeof(char)*(m+1));
+        if(read(fd, buf, m) < 0) {
+            perror("read");
+            exit(1);
+        }
+        buf[m] = 0;
+
+        self->mBody = STRING_NEW(buf);
+        }
+        break;
+
+    case 8:
+        self->mBody = sStatments_load(fd);
+        break;
+
     }
 
     return self;
 }
 
 //////////////////////////////////////////////////////////
-// 変数
+// プロセス置換
 //////////////////////////////////////////////////////////
+sPSub* sPSub_new(char* fname, sStatments* statments)
+{
+    sPSub* self = MALLOC(sizeof(sPSub));
+
+    self->mFName = STRING_NEW(fname);
+    self->mStatments = statments;
+
+    return self;
+}
 
 //////////////////////////////////////////////////////////
 // sSubshell
@@ -1267,6 +1367,10 @@ sCommand* sCommand_new2(sCommand* command)
         self->mExtra = sFunction_new2(command->mExtra);
         break;
 
+    case kClass:
+        self->mExtra = sClass_new2(command->mExtra);
+        break;
+
     default:
         self->mExtra = NULL;
         break;
@@ -1312,6 +1416,10 @@ void sCommand_view(sCommand* self)
     case kDef:
         sFunction_view(self->mExtra);
         break;
+
+    case kClass:
+        sClass_view(self->mExtra);
+        break;
     }
 }
 
@@ -1372,6 +1480,10 @@ void sCommand_save(sCommand* self, int fd)
     case kDef:
         sFunction_save(self->mExtra, fd);
         break;
+        
+    case kClass:
+        sClass_save(self->mExtra, fd);
+        break;
     }
 }
 
@@ -1434,6 +1546,10 @@ sCommand* sCommand_load(int fd)
         self->mExtra = sFunction_load(fd);
         break;
         
+    case kClass:
+        self->mExtra = sClass_load(fd);
+        break;
+        
     default:
         self->mExtra = NULL;
     }
@@ -1833,13 +1949,82 @@ sStatments* sStatments_load(int fd)
     return self;
 }
 
+/////////////////////////////////////////////////////////
+// クラス
+/////////////////////////////////////////////////////////
+sClass* sClass_new(char* name, sStatments* statments)
+{
+    sClass* self = MALLOC(sizeof(sClass));
+    self->name = STRING_NEW(name);
+    self->statments = statments;
+
+    return self;
+}
+
+sClass* sClass_new2(sClass* klass)
+{
+    sClass* self = MALLOC(sizeof(sClass));
+    self->name = STRING_NEW(string_c_str(klass->name));
+    
+    if(klass->statments) {
+        self->statments = sStatments_new2(klass->statments);
+    }
+    else {
+        self->statments = NULL;
+    }
+
+    return self;
+}
+
+void sClass_save(sClass* self, int fd)
+{
+    int n = string_length(self->name);
+    if(write(fd, &n, sizeof(int)) < 0) {
+        perror("write");
+        exit(1);
+    }
+    if(write(fd, string_c_str(self->name), sizeof(char)*n) < 0) {
+        perror("write");
+        exit(1);
+    }
+
+    sStatments_save(self->statments, fd);
+}
+
+sClass* sClass_load(int fd)
+{
+    sClass* self = MALLOC(sizeof(sClass));
+    
+    int n;
+    if(read(fd, &n, sizeof(int)) < 0) {
+        perror("read");
+        exit(1);
+    }
+
+    char* buf = MALLOC(sizeof(char)*(n+1));
+    if(read(fd, buf, sizeof(char)*n) < 0) {
+        perror("read");
+        exit(1);
+    }
+    buf[n] = 0;
+
+    self->name = STRING_NEW(buf);
+
+    self->statments = sStatments_load(fd);
+
+    return self;
+}
+
+void sClass_view(sClass* self)
+{
+    printf("name %s\n", string_c_str(self->name));
+}
 
 /////////////////////////////////////////////////////////
 // 関数
 /////////////////////////////////////////////////////////
 sFunction* sFunction_new(char* name, MANAGED sStatments* statments, char* arg_name)
 {
-
     sFunction* self = MALLOC(sizeof(sFunction));
     self->name = STRING_NEW(name);
     self->arg_name = STRING_NEW(arg_name);
@@ -1851,7 +2036,6 @@ sFunction* sFunction_new(char* name, MANAGED sStatments* statments, char* arg_na
 
 sFunction* sFunction_new2(sFunction* func)
 {
-
     sFunction* self = MALLOC(sizeof(sFunction));
     self->name = STRING_NEW(string_c_str(func->name));
     self->arg_name = STRING_NEW(string_c_str(func->arg_name));
@@ -1934,21 +2118,6 @@ void sFunction_view(sFunction* self)
     printf("arg_name %s\n", string_c_str(self->arg_name));
 }
 
-
-///////////////////////////////////////////////////////
-// 子プロセス
-///////////////////////////////////////////////////////
-sChildProgram* sChildProgram_new(pid_t pid, char* title, BOOL writer_process)
-{
-    sChildProgram* self = (sChildProgram*)MALLOC(sizeof(sChildProgram));
-
-    self->mPid = pid;
-    self->mTitle = STRING_NEW(title);
-    self->mWriterProcess = writer_process;
-
-    return self;
-}
-
 //////////////////////////////////////////////////////
 // saphireが組み込まれたアプリケーションから
 // 登録された内部コマンド
@@ -1963,7 +2132,6 @@ sInnerCommand* sInnerCommand_new(char* name, fInnerCommand fun)
     return self;
 }
 
-
 /////////////////////////////////////////////////////////
 // プログラム名補完候補を再読み込み
 /////////////////////////////////////////////////////////
@@ -1975,27 +2143,27 @@ static BOOL name_sort(void* left, void* right)
     return strcmp(string_c_str(l), string_c_str(r)) < 0;
 }
 
-static void saphire_rehash_without_user_fun()
+static void saphire_rehash_external_prog()
 {
     hash_obj* check = HASH_NEW(3000);
 
-    vector_clear(gSaphireProgCompletions);
+    vector_clear(gSaphireExternalProgCompletions);
 
     /// 内部コマンド ///
     int i;
     for(i=0; i<kCommand; i++) {
-        vector_add(gSaphireProgCompletions, STRING_NEW(gStatmentKindStrs[i]));
+        vector_add(gSaphireExternalProgCompletions, STRING_NEW(gStatmentKindStrs[i]));
     }
 
     /// 外部登録の内部コマンド ///
     hash_it* it = hash_loop_begin(gInnerCommands);
     while(it != NULL) {
         sInnerCommand* command = hash_loop_item(it);
-        vector_add(gSaphireProgCompletions, STRING_NEW(string_c_str(command->mName)));
+        vector_add(gSaphireExternalProgCompletions, STRING_NEW(string_c_str(command->mName)));
         it = hash_loop_next(it);
     }
 
-    /// $PATHにあるファイルを全てgSaphireProgCompletionsに入れていく ///
+    /// $PATHにあるファイルを全てgSaphireExternalProgCompletionsに入れていく ///
     char* path = getenv("PATH");
     if(path == NULL) {
         fprintf(stderr, "$PATH is NULL\n");
@@ -2042,15 +2210,15 @@ static void saphire_rehash_without_user_fun()
                                 memcpy(buf, entry->d_name
                                         , strlen(entry->d_name) -4);
                                 buf[strlen(entry->d_name) -4] = 0;
-                                vector_add(gSaphireProgCompletions, STRING_NEW(buf));
+                                vector_add(gSaphireExternalProgCompletions, STRING_NEW(buf));
                             }
                             else {
-                                vector_add(gSaphireProgCompletions
+                                vector_add(gSaphireExternalProgCompletions
                                         , STRING_NEW(entry->d_name));
                             }
                             
 #else
-                            vector_add(gSaphireProgCompletions
+                            vector_add(gSaphireExternalProgCompletions
                                         , STRING_NEW(entry->d_name));
 #endif
                         }
@@ -2074,14 +2242,14 @@ static void saphire_rehash_without_user_fun()
             if(strcmp(entry->d_name, ".") != 0
                     && strcmp(entry->d_name, "..") != 0)
             {
-                vector_add(gSaphireProgCompletions, STRING_NEW(entry->d_name));
+                vector_add(gSaphireExternalProgCompletions, STRING_NEW(entry->d_name));
             }
         }
 
         closedir(dir);
     }
 
-    (void)vector_sort(gSaphireProgCompletions, name_sort);
+    (void)vector_sort(gSaphireExternalProgCompletions, name_sort);
 }
 
 static void saphire_rehash_user_fun()
@@ -2095,10 +2263,59 @@ static void saphire_rehash_user_fun()
     }
 }
 
+void saphire_rehash_object()
+{
+    /// ユーザーオリジナル関数 ///
+    hash_it* it = hash_loop_begin(gObjects);
+    while(it != NULL) {
+        sObject* object = hash_loop_item(it);
+        string_obj* str = STRING_NEW(string_c_str(object->mName));
+        string_push_back(str, "->");
+        vector_add(gSaphireProgCompletions, str);
+        it = hash_loop_next(it);
+    }
+}
+
+void saphire_rehash_class()
+{
+    /// ユーザーオリジナル関数 ///
+    hash_it* it = hash_loop_begin(gClasses);
+    while(it != NULL) {
+        sClass* klass = hash_loop_item(it);
+        string_obj* str = STRING_NEW(string_c_str(klass->name));
+        vector_add(gSaphireProgCompletions, str);
+        it = hash_loop_next(it);
+    }
+}
+
 void saphire_rehash()
 {
-    saphire_rehash_without_user_fun();
+    saphire_rehash_external_prog();
+
+    vector_clear(gSaphireProgCompletions);
+
+    int i;
+    for(i=0; i< vector_size(gSaphireExternalProgCompletions); i++) {
+        vector_add(gSaphireProgCompletions, vector_item(gSaphireExternalProgCompletions, i));
+    }
+
     saphire_rehash_user_fun();
+    saphire_rehash_object();
+    saphire_rehash_class();
+}
+
+void saphire_rehash2()
+{
+    vector_clear(gSaphireProgCompletions);
+
+    int i;
+    for(i=0; i< vector_size(gSaphireExternalProgCompletions); i++) {
+        vector_add(gSaphireProgCompletions, vector_item(gSaphireExternalProgCompletions, i));
+    }
+
+    saphire_rehash_user_fun();
+    saphire_rehash_object();
+    saphire_rehash_class();
 }
 
 ///////////////////////////////////////////////////////
@@ -2216,14 +2433,17 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gArrays = HASH_NEW(100);
     gHashs = HASH_NEW(100);
     gFuncs = HASH_NEW(50);
+    gClasses = HASH_NEW(50);
 
     gJobs = VECTOR_NEW(50);
 
     gGlobals = HASH_NEW(100);
 
+    gObjects = HASH_NEW(50);
+
     gReadBuffers = HASH_NEW(100);
 
-    setenv("SAPHIRE_VERSION", "3.1.3", 1);
+    setenv("SAPHIRE_VERSION", "3.2.0", 1);
     setenv("SAPHIRE_PROMPT", "> ", 1);
     setenv("SAPHIRE_DOCDIR", DOCDIR, 1);
     char* home = getenv("HOME");
@@ -2241,6 +2461,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     }
     
     gSaphireProgCompletions = VECTOR_NEW(100);
+    gSaphireExternalProgCompletions = VECTOR_NEW(100);
 
     gRegexs = HASH_NEW(100);
     gRegexsI = HASH_NEW(100);
@@ -2253,7 +2474,7 @@ void saphire_init(enum eAppType app_type, BOOL job_control, enum eRuntimeScript
     gInnerCommands = HASH_NEW(50);
     gKitutukiExit = -1;
 
-    gPSubTmpFiles = VECTOR_NEW(10);
+    gTmpFiles = VECTOR_NEW(10);
     gDirStack = VECTOR_NEW(10);
     gErrMsg = STRING_NEW("");
 
@@ -2328,8 +2549,8 @@ void saphire_final()
     
     /// プロセス置換で使った一時ファイルを掃除 ///
     int k;
-    for(k=0; k<vector_size(gPSubTmpFiles); k++) {
-        string_obj* fname = vector_item(gPSubTmpFiles, k);
+    for(k=0; k<vector_size(gTmpFiles); k++) {
+        string_obj* fname = vector_item(gTmpFiles, k);
 
         if(unlink(string_c_str(fname)) < 0) {
             perror("unlink");
@@ -3092,6 +3313,9 @@ void saphire_make_var_completion_list()
     (void)vector_sort(gSaphireVarCompletions, name_sort);
 }
 
+/////////////////////////////////////////////////////////////////////////
+// ブロック
+/////////////////////////////////////////////////////////////////////////
 sBlock* sBlock_new(sStatments* statments, vector_obj* args, int count)
 {
     sBlock* self = MALLOC(sizeof(sBlock));
@@ -3193,3 +3417,22 @@ void sBlock_save(sBlock* self, int fd)
     }
 }
 
+vector_obj* saphire_methods_names(sObject* object)
+{
+    vector_obj* v = VECTOR_NEW(50);
+    hash_it* it = hash_loop_begin(object->mMethods);
+    while(it) {
+        sFunction* fun = hash_loop_item(it);
+
+        vector_add(v, STRING_NEW(string_c_str(fun->name)));
+
+        it = hash_loop_next(it);
+    }
+
+    return v;
+}
+
+sObject* saphire_get_object(char* name)
+{
+    return hash_item(gObjects, name);
+}
