@@ -361,7 +361,7 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
     static unsigned char table[] = { 
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0,
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
-    1, 0, 0, 0, 0, 1, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 
+    1, 0, 0, 0, 0, 0, 1, 0, 1, 1, 1, 0, 0, 0, 0, 0, 
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 1, 1, 
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
     0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 
@@ -498,7 +498,7 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
             }
         }
         /// グロブ ///
-        else if(**p == '*' || command && vector_size(command->mArgs) > 0 && **p == '[' || **p == '?') {
+        else if(command && vector_size(command->mArgs) > 0 && (**p == '[' || **p == '?' || **p == '*')) {
             string_push_back2(not_evaled, **p);
             string_push_back2(buf, **p);
             (*p)++;
@@ -543,6 +543,22 @@ static BOOL read_one_argument(char** p, string_obj* buf, string_obj* not_evaled,
         else if(**p == '@' && *(*p+1) == '(' || **p == '@' && *(*p+1) == '@' && *(*p+2) == '(') {
             break;
         }
+        // % のりダイレクトは下で処理
+        else if(**p == '%' && (*(*p+1) == '>' || *(*p+1) == '2')) {
+            break;
+        }
+        // ->は下で処理
+        else if(command && vector_size(command->mArgs) == 0 && **p == '-' && *(*p+1) == '>') {
+            string_push_back2(not_evaled, **p);
+            string_push_back2(buf, **p);
+
+            (*p)++;
+
+            string_push_back2(not_evaled, **p);
+            string_push_back2(buf, **p);
+
+            (*p)++;
+        }
         else if(table[**p]) {
             break;
         }
@@ -958,6 +974,65 @@ static BOOL read_command_parse_def_statment(char** p, sCommand* command, sStatme
     return TRUE;
 }
 
+/// class文読み込み
+static BOOL read_command_parse_class_statment(char** p, sCommand* command, sStatment* statment, string_obj* not_evaled, int* read_end_of_statment, char* sname, int* sline)
+{
+    skip_spaces(p, not_evaled);
+
+    /// クラス名を得る ///
+    string_obj* name = STRING_NEW("");
+    while(**p >= 'a' && **p <= 'z' 
+            || **p >= 'A' && **p <= 'Z'
+            || **p == '_' 
+            || **p >= '0' && **p <= '9')
+    {
+        string_push_back2(not_evaled, **p);
+        string_push_back2(name, **p);
+        (*p)++;
+    }
+
+    while(**p) {
+        if(**p == '\n') {
+            (*sline)++;
+            string_push_back2(not_evaled, **p);
+            (*p)++;
+        }
+        else if(**p == ' ' || **p == '\t') {
+            string_push_back2(not_evaled, **p);
+            (*p)++;
+        }
+        else {
+            break;
+        }
+    }
+
+    string_obj* contents = STRING_NEW("");
+    if(**p == '{') {
+        if(!read_block(p, contents, NULL, not_evaled, FALSE, sname, sline)) {
+            return FALSE;
+        }
+    }
+    else {
+        err_msg("unexpected token at def statment", sname, *sline);
+        return FALSE;
+    }
+
+    sStatments* statments = STATMENTS_NEW();
+
+    if(!parse(string_c_str(contents), sname, sline, statments)) 
+    {
+
+        return FALSE;
+    }
+
+    command->mKind = kClass;
+        // これが無いとmNotEvaledの場合（変数がある場合）
+        // 正しくcommandが消されない
+    command->mExtra = sClass_new(string_c_str(name), statments);
+
+    return TRUE;
+}
+
 /// １コマンド読み込み
 static BOOL read_command(char** p, sCommand* command, sStatment* statment, string_obj* not_evaled, int* read_end_of_statment, BOOL block_termination, char* sname, int* sline)
 {
@@ -1011,6 +1086,16 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                         return FALSE;
                     }
                 }
+                else if(strcmp(string_c_str(buf), "class") == 0) {
+                    vector_add(command->mArgs, sArg_new(0, buf));
+                    string_push_back(statment->mTitle, string_c_str(buf));
+                    string_push_back(statment->mTitle, " ");
+
+                    if(!read_command_parse_class_statment(p, command, statment, not_evaled, read_end_of_statment, sname, sline))
+                    {
+                        return FALSE;
+                    }
+                }
                 else {
                     if(vector_size(statment->mCommands) == 0 && strcmp(string_c_str(buf), "!") == 0)
                     {
@@ -1089,18 +1174,27 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
                 vector_add(command->mArgs, sArg_new(2, statments));
             }
         }
-        /// リダイレクト・プロセス置換
         else if(**p == '>') {
             /// プロセス置換 ///
             if(*(*p+1) == '(') {
                 string_push_back2(not_evaled, **p);
                 (*p)++;
 
-                if(!read_until_next_close_character('(', ')', p, not_evaled, read_end_of_statment, TRUE, sname, sline)) {
+                string_obj* cmd = STRING_NEW("");
+                if(!read_until_next_close_character('(', ')', p, cmd, read_end_of_statment, FALSE, sname, sline)) {
                     return FALSE;
                 }
-                
-                *read_end_of_statment = 1;
+                string_push_back(not_evaled, string_c_str(cmd));
+
+                string_erase(cmd, 0, 1); // (を取る
+                string_trunc(cmd, string_length(cmd)-1); // )をとる
+
+                sStatments* statments = STATMENTS_NEW();
+                if(!parse(string_c_str(cmd), sname, sline, statments)) {
+                    return FALSE;
+                }
+
+                vector_add(command->mArgs, sArg_new(8, statments));
             }
             else if(*(*p+1) == '>') {
                 string_push_back(not_evaled, ">>");
@@ -1151,10 +1245,144 @@ static BOOL read_command(char** p, sCommand* command, sStatment* statment, strin
 
             }
         }
-        /// リダイレクト・プロセス置換 ///
         else if(**p == '<') {
+            /// ヘアドキュメント
+            if(*(*p+1) == '<') {
+                string_push_back2(not_evaled, **p);
+                (*p)++;
+                string_push_back2(not_evaled, **p);
+                (*p)++;
+
+                skip_spaces(p, not_evaled);
+
+                BOOL expand_env = TRUE;
+
+                BOOL squote = FALSE;
+                BOOL dquote = FALSE;
+                if(**p == '\'') {
+                    squote = TRUE;
+                    string_push_back2(not_evaled, **p);
+                    (*p)++;
+                }
+                else if(**p == '"') {
+                    dquote = TRUE;
+                    string_push_back2(not_evaled, **p);
+                    (*p)++;
+                }
+
+                string_obj* name = STRING_NEW("");
+                while(**p) {
+                    if(**p == '\n') {
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+                        break;
+                    }
+                    else if(**p == ' ' || **p == '\t') {
+                        skip_spaces(p, not_evaled);
+                        if(**p != '\n') {
+                            err_msg("invalid here document name", sname, *sline);
+                            return FALSE;
+                        }
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+                        break;
+                    }
+                    else if(**p >='A' && ** p <= 'Z' || **p >='a' && **p <='z')
+                    {
+                        string_push_back2(name, **p);
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+                    }
+                    else if(dquote && **p == '"') {
+                        expand_env = TRUE;
+                        dquote = FALSE;
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+                        skip_spaces(p, not_evaled);
+                        if(**p != '\n') {
+                            err_msg("invalid here document name", sname, *sline);
+                            return FALSE;
+                        }
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+                        break;
+                    }
+                    else if(squote && **p == '\'') {
+                        expand_env = FALSE;
+                        squote = FALSE;
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+                        skip_spaces(p, not_evaled);
+                        if(**p != '\n') {
+                            err_msg("invalid here document name", sname, *sline);
+                            return FALSE;
+                        }
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+                        break;
+                    }
+                    else {
+                        err_msg("invalid here document name", sname, *sline);
+                        return FALSE;
+                    }
+                }
+
+                if(dquote || squote) {
+                    err_msg("invalid here document name", sname, *sline);
+                    return FALSE;
+                }
+                
+                if(string_c_str(name)[0] == 0) {
+                    err_msg("invalid here document name", sname, *sline);
+                    return FALSE;
+                }
+
+                string_obj* text = STRING_NEW("");
+                string_obj* line = STRING_NEW("");
+                while(**p) {
+                    if(**p == '\n') {
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+                        string_push_back(text, string_c_str(line));
+                        string_push_back2(text, '\n');
+                        string_put(line, "");
+                    }
+                    /// コマンド展開 ///
+                    else if(expand_env && **p == '$' && *(*p+1) == '(' || **p == '$' && *(*p+1) == '$' && *(*p+2) == '(') 
+                    {
+                        if(*(*p+1) == '$') {
+                            string_push_back2(not_evaled, **p); // $
+                            (*p)++;
+                            string_push_back2(not_evaled, **p); // $
+                            (*p)++;
+                        }
+                        else {
+                            string_push_back2(not_evaled, **p); // $
+                            (*p)++;
+                        }
+
+                        if(!read_until_next_close_character('(', ')', p, not_evaled, read_end_of_statment, TRUE, sname, sline)) {
+                            return FALSE;
+                        }
+
+                        *read_end_of_statment = 1;
+                    }
+                    else {
+                        string_push_back2(line, **p);
+                        string_push_back2(not_evaled, **p);
+                        (*p)++;
+
+                        if(strstr(string_c_str(line), string_c_str(name)) == string_c_str(line))
+                        {
+                            break;
+                        }
+                    }
+                }
+
+                vector_add(command->mArgs, sArg_new(7, text));
+            }
             /// プロセス置換 ///
-            if(*(*p+1) == '(') {
+            else if(*(*p+1) == '(') {
                 string_push_back2(not_evaled, **p);
                 (*p)++;
 
@@ -1610,7 +1838,7 @@ static BOOL expand_env_get_quoted_str(char* fname, string_obj* quoted_fname, cha
                 || *p=='\\' || *p=='|' || *p=='[' || *p==']' || *p=='{' 
                 || *p=='}' || *p==';'
                 || *p=='\'' || *p=='"' || *p=='<' || *p=='>' || *p=='?' 
-                || *p=='%')
+                || *p=='%' || *p=='@')
         {
             string_push_back2(quoted_fname, '\\');
             string_push_back2(quoted_fname, *p++);
@@ -1623,11 +1851,10 @@ static BOOL expand_env_get_quoted_str(char* fname, string_obj* quoted_fname, cha
     return TRUE;
 }
 
-BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline)
+BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline, sObject* object)
 {
     BOOL squote = FALSE;
     BOOL dquote = FALSE;
-    vector_obj* psub_statments = VECTOR_NEW(10);
 
     while(*p) {
         if(gKitutukiSigInt) {
@@ -1684,7 +1911,7 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline)
                 string_obj* ret = STRING_NEW("");
                 char* fname = MALLOC(strlen(sname) + 256);
                 snprintf(fname, strlen(sname) + 256, "%s %d: expand command", sname, *sline);
-                int rcode = saphire_shell3(ret, string_c_str(str), fname, rfd);
+                int rcode = saphire_shell5(ret, string_c_str(str), fname, rfd, object);
                 if(rcode < 0) {
                     return FALSE;
                 }
@@ -1714,33 +1941,6 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline)
             string_push_back2(out, *p++);
         }
 
-        /// プロセス置換２ ///
-        else if(*p == '>' && *(p+1) == '(') {
-            p++;
-
-            string_obj* cmd = STRING_NEW("");
-            int read_end_of_statment = -1;
-            if(!read_until_next_close_character('(', ')', &p, cmd, &read_end_of_statment, FALSE, sname, sline)) {
-                return FALSE;
-            }
-            string_erase(cmd, 0, 1);
-            string_trunc(cmd, string_length(cmd)-1); // )をとる
-
-            /// 一時ファイルを作る ///
-            char fname[PATH_MAX];
-            snprintf(fname, PATH_MAX, "%s/kifile%d", gTmpDir, vector_size(gPSubTmpFiles));
-
-            string_push_back(out, fname);
-            vector_add(gPSubTmpFiles, STRING_NEW(fname));
-
-            /// 実行 ///
-            char buf[PATH_MAX*2];
-            snprintf(buf, PATH_MAX*2, "; var RCODE |var __a; cat %s |", fname);
-            string_insert(cmd, 0, buf);
-            string_push_back(cmd, "; if([ $(var __a) = 0 ]){ true } else { false };");
-
-            vector_add(psub_statments, cmd);
-        }
         /// ブロックは飛ばす
         else if(*p == '{') {
             string_obj* dummy = STRING_NEW("");
@@ -1761,12 +1961,6 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* sline)
         }
     }
 
-    int i;
-    for(i=0; i<vector_size(psub_statments); i++) {
-        string_obj* psub_statment = vector_item(psub_statments, i);
-        string_push_back(out, string_c_str(psub_statment));
-    }
-
     return TRUE;
 }
 
@@ -1912,13 +2106,13 @@ int saphire_load_obj(char* fname, sWFd* pipeout, sRFd* pipein, int pipeerr, vect
 
     /// 実行 ///
     int return_ = FALSE;
+    vector_obj* envs = VECTOR_NEW(10);
     int rcode = run(statments, fname
                 , pipeout, pipein, pipeerr
-                , &return_, parent_blocks, FALSE);
+                , &return_, parent_blocks, FALSE, envs, NULL);
 
     /// 解放 ///
     close(fd);
 
     return rcode;
 }
-
