@@ -230,7 +230,7 @@ static BOOL statment_tree_exec_cprog(sCommand* command, int nextin, int nextout,
 }
 
 // 外部コマンドの処理
-static BOOL statment_tree_external_command(sStatment* statment, vector_obj* argv, sRFd* nextin, sWFd* nextout, int nexterr, sWFd* pipeout, sCommand* command, BOOL last_program, int* last_program_pid, char* sname, int sline)
+static BOOL statment_tree_external_command(sStatment* statment, vector_obj* argv, sRFd* nextin, sWFd* nextout, int nexterr, sWFd* pipeout, sCommand* command, BOOL last_program, int* last_program_pid, char* sname, int sline, vector_obj* envs)
 {
 //printf("nextin %d nextout %d\n", nextin->mFd, nextout->mFd);
     char* argv0 = string_c_str(vector_item(argv, 0));
@@ -267,6 +267,15 @@ static BOOL statment_tree_external_command(sStatment* statment, vector_obj* argv
 
     /// 子プロセスの処理 ///
     if(pid == 0) {
+        /// 環境変数 ///
+        int i;
+        for(i=0; i<vector_size(envs); i+=2) {
+            char* name = string_c_str(vector_item(envs, i));
+            char* value = string_c_str(vector_item(envs, i+1));
+
+            setenv(name, value, 1);
+        }
+        
         if(mpipefds[0] != -1) (void)close(mpipefds[0]);
         if(mpipefds[1] != -1) (void)close(mpipefds[1]);
         if(mpipefds2[0] != -1) (void)close(mpipefds2[0]);
@@ -494,7 +503,7 @@ static BOOL statment_tree_external_command(sStatment* statment, vector_obj* argv
 }
 
 /// argvを作る
-static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, BOOL* return_, sRFd* nextin, sRFd* pipein, char* sname, int sline)
+static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, BOOL* return_, sRFd* nextin, sRFd* pipein, char* sname, int sline, vector_obj* envs, vector_obj* psubs, sObject* object)
 {
     int ii;
     for(ii=0; ii<vector_size(command->mArgs); ii++) {
@@ -510,7 +519,8 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 sStatments* statments = arg->mBody;
                 sWFd* pipeout = sWFd_new(-1);
                 int rcode = run(statments, "quick command expansion"
-                    , pipeout, pipein, 2, return_, parent_blocks, FALSE);
+                    , pipeout, pipein, 2, return_, parent_blocks, FALSE
+                    , envs, object);
 
                 if(rcode == -1) {
                     return FALSE;
@@ -528,7 +538,8 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 sStatments* statments = arg->mBody;
                 sWFd* pipeout = sWFd_new(-1);
                 int rcode = run(statments, "quick command expansion"
-                    , pipeout, pipein, 2, return_, parent_blocks, FALSE);
+                    , pipeout, pipein, 2, return_, parent_blocks, FALSE
+                    , envs, object);
 
                 if(rcode == -1) {
                     return FALSE;
@@ -567,7 +578,8 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 sStatments* statments = arg->mBody;
                 sWFd* pipeout = sWFd_new(-1);
                 int rcode = run(statments, "process substitution"
-                    , pipeout, pipein, 2, return_, parent_blocks, FALSE);
+                    , pipeout, pipein, 2, return_, parent_blocks, FALSE
+                    , envs, object);
 
                 if(rcode == -1) {
                     return FALSE;
@@ -580,11 +592,11 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 char fname[PATH_MAX];
                 while(1) {
                     int n = random() % 1000;
-                    snprintf(fname, PATH_MAX, "%s/kifile%d", gTmpDir, n);
+                    snprintf(fname, PATH_MAX, "%s/psub2%d", gTmpDir, n);
                     int flg = 0;
                     int i;
-                    for(i=0; i<vector_size(gPSubTmpFiles); i++) {
-                        char* fname2 = string_c_str(vector_item(gPSubTmpFiles, i));
+                    for(i=0; i<vector_size(gTmpFiles); i++) {
+                        char* fname2 = string_c_str(vector_item(gTmpFiles, i));
                         if(strcmp(fname, fname2) == 0) {
                             flg++;
                         }
@@ -592,7 +604,7 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
 
                     if(flg == 0) break;
                 }
-                vector_add(gPSubTmpFiles, STRING_NEW(fname));
+                vector_add(gTmpFiles, STRING_NEW(fname));
                 
                 int fd = open(fname, O_WRONLY|O_TRUNC|O_CREAT, 0600);
                 if(write(fd, pipeout->mBuffer, strlen(pipeout->mBuffer)) < 0)
@@ -606,6 +618,67 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
                 vector_add(argv, STRING_NEW(fname));
                 }
                 break;
+
+            /// ヘアドキュメント ///
+            case 7: {
+                string_obj* text = arg->mBody;
+
+                /// 一時ファイルを作る ///
+                char fname[PATH_MAX];
+                while(1) {
+                    int n = random() % 1000;
+                    snprintf(fname, PATH_MAX, "%s/here_doc%d", gTmpDir, n);
+                    int flg = 0;
+                    int i;
+                    for(i=0; i<vector_size(gTmpFiles); i++) {
+                        char* fname2 = string_c_str(vector_item(gTmpFiles, i));
+                        if(strcmp(fname, fname2) == 0) {
+                            flg++;
+                        }
+                    }
+
+                    if(flg == 0) break;
+                }
+
+                vector_add(gTmpFiles, STRING_NEW(fname));
+
+                int f = open(fname, O_WRONLY|O_TRUNC|O_CREAT, 0644);
+                if(write(f, string_c_str(text), string_length(text)) < 0) {
+                    perror("write on here document");
+                    exit(1);
+                }
+                (void)close(f);
+
+                vector_add(argv, STRING_NEW(fname));
+                }
+                break;
+
+            /// プロセス置換2 ///
+            case 8: {
+                sStatments* statments = arg->mBody;
+
+                /// 一時ファイルを作る ///
+                char fname[PATH_MAX];
+                while(1) {
+                    int n = random() % 1000;
+                    snprintf(fname, PATH_MAX, "%s/psub%d", gTmpDir, n);
+                    int flg = 0;
+                    int i;
+                    for(i=0; i<vector_size(gTmpFiles); i++) {
+                        char* fname2 = string_c_str(vector_item(gTmpFiles, i));
+                        if(strcmp(fname, fname2) == 0) {
+                            flg++;
+                        }
+                    }
+
+                    if(flg == 0) break;
+                }
+                vector_add(gTmpFiles, STRING_NEW(fname));
+                vector_add(argv, STRING_NEW(fname));
+                vector_add(psubs, sPSub_new(fname, statments));
+                }
+                break;
+
         }
     }
 
@@ -616,7 +689,7 @@ static BOOL statment_tree_make_argv(sCommand* command, vector_obj* argv, vector_
 // 文を回す
 // TRUE 正常終了
 // FALSE エラー
-static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipeout, sRFd* pipein, int pipeerr, char* title, int* rcode, BOOL* return_, vector_obj* parent_blocks, char* sname, int sline) 
+static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipeout, sRFd* pipein, int pipeerr, char* title, int* rcode, BOOL* return_, vector_obj* parent_blocks, char* sname, int sline, vector_obj* envs, vector_obj* psubs, sObject* object) 
 {
     sRFd* nextin;
     int nexterr = pipeerr;
@@ -679,7 +752,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
         // argvを作成 sArgから
         vector_obj* argv = VECTOR_NEW(30);
         vector_obj* blocks = VECTOR_NEW(30);
-        if(!statment_tree_make_argv(command, argv, blocks, parent_blocks, return_, nextin, pipein, sname, sline)) {
+        if(!statment_tree_make_argv(command, argv, blocks, parent_blocks, return_, nextin, pipein, sname, sline, envs, psubs, object)) {
             return FALSE;
         }
         if(*return_) {
@@ -689,15 +762,243 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
         int ii;
 
         char* arg0 = string_c_str(vector_item(argv, 0));
-        sFunction* fun = hash_item(gFuncs, arg0);
-        sInnerCommand* inner_fun = hash_item(gInnerCommands, arg0);
+        sObject* object2;
+        sFunction* method;
+        sFunction* fun;
+        sClass* klass;
+        sInnerCommand* inner_fun;
+        char* tmp = strstr(arg0, "->");
+        if(tmp) {
+            char* tmp2 = MALLOC(tmp - arg0 + 1);
+            memcpy(tmp2, arg0, tmp-arg0);
+            tmp2[tmp - arg0] = 0;
+
+            if(object) {
+                if(strcmp(tmp2, "self") == 0) {
+                    object2 = object;
+                }
+                else {
+                    object2 = hash_item(object->mObjects, tmp2);
+                }
+            }
+            else {
+                object2 = hash_item(gObjects, tmp2);
+            }
+
+            if(object2 == NULL) {
+                char str[1024];
+                snprintf(str, 1024, "object(%s) is not found", tmp2);
+                err_msg(str, sname, sline);
+                return FALSE;
+            }
+
+            tmp += 2;
+            char* tmp3 = MALLOC(arg0 + strlen(arg0) - tmp + 1);
+            strcpy(tmp3, tmp);
+
+            method = hash_item(object2->mMethods, tmp3);
+
+            if(method == NULL) {
+                char str[1024];
+                snprintf(str, 1024, "method(%s) is not found", tmp3);
+                err_msg(str, sname, sline);
+                return FALSE;
+            }
+            fun = NULL;
+            inner_fun = NULL;
+            klass = NULL;
+        }
+        else {
+            object2 = NULL;
+            method = NULL;
+            fun = hash_item(gFuncs, arg0);
+            inner_fun = hash_item(gInnerCommands, arg0);
+            if(object) {
+                klass = hash_item(object->mClasses, arg0);
+                if(klass == NULL) {
+                    sObject* parent = object->mParent;
+                    while(parent) {
+                        klass = hash_item(parent->mClasses, arg0);
+                        if(klass) break;
+                        parent = parent->mParent;
+                    }
+                }
+                if(klass == NULL) {
+                    klass = hash_item(gClasses, arg0);
+                }
+            }
+            else {
+                klass = hash_item(gClasses, arg0);
+            }
+        }
 
         if(fun || inner_fun) {
             command->mKind = kInnerCommand;
         }
 
+        /// クラス ///
+        if(klass) {
+            if(vector_size(argv) >= 2) {
+                char* objname = string_c_str(vector_item(argv, 1));
+
+                /// スタックフレームの記録を付ける
+                vector_add(gStackTraceFName, STRING_NEW(sname));
+                vector_add(gStackTraceLineNum, (void*)sline);
+
+                // スタックを掘る
+                vector_add(gStackFrame, HASH_NEW(30));
+
+                sObject* object2 = sObject_new(objname, object);
+                if(object) {
+                    hash_put(object->mObjects, objname, object2);
+                }
+                else {
+                    hash_put(gObjects, objname, object2);
+                }
+
+                *rcode = run(klass->statments, title, nextout
+                                , nextin, nexterr
+                                , return_, parent_blocks, FALSE, envs, object2);
+                *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+
+                // スタックを埋める
+                vector_pop_back(gStackFrame);
+
+                /// スタックフレームの記録を消す
+                string_obj* sname_strace = vector_pop_back(gStackTraceFName);
+                int line_strace = (int)vector_pop_back(gStackTraceLineNum);
+
+                /// 終了コード ///
+                if(statment->mRCodeReverse) {
+                    if(*rcode == 0) 
+                        *rcode = 1;
+                    else
+                        *rcode = 0;
+                }
+
+                char buf2[256];
+                snprintf(buf2, 256, "%d", *rcode);
+                saphire_set_local_var("RCODE", buf2);
+
+                if(*rcode < 0) {
+                    string_push_back(gErrMsg, string_c_str(sname_strace));
+                    char tmp[1024];
+                    snprintf(tmp, 1024, ":%d\n", line_strace);
+                    string_push_back(gErrMsg, tmp);
+
+                    return FALSE;
+                }
+            }
+            else {
+                err_msg("reuire object name", sname, sline);
+                return FALSE;
+            }
+        }
+        /// メソッド ///
+        else if(method) {
+            if(!statment_tree_redirect(command, &nextin, &nextout
+                            , &nexterr, pipein, pipeout, pipeerr, sname, sline))
+            {
+                return FALSE;
+            }
+
+            /// 引数名が指定されていなければスタックフレームを増やす ///
+            vector_obj* fun_argv_before;
+            vector_obj* fun_argv;
+            if(strcmp(string_c_str(method->arg_name), "") == 0) {
+                /// 前の引数を保存 ///
+                fun_argv_before = hash_item(gArrays, "ARGV");
+
+                /// 引数を渡す ///
+                int k;
+                fun_argv = VECTOR_NEW(30);
+                for(k=1; k<vector_size(argv); k++) {
+                    char* value = string_c_str(vector_item(argv, k));
+                    vector_add(fun_argv, STRING_NEW(value));
+                }
+                
+                hash_put(gArrays, "ARGV", fun_argv);
+
+                vector_add(gStackFrame, HASH_NEW(30));
+            }
+            /// 引数名が指定されていなければスタックフレームは増やさない
+            else {
+                /// 前の引数を保存 ///
+                fun_argv_before = hash_item(gArrays
+                                        , string_c_str(method->arg_name));
+
+                /// 引数を渡す ///
+                int k;
+                fun_argv = VECTOR_NEW(30);
+                for(k=1; k<vector_size(argv); k++) {
+                    char* value = string_c_str(vector_item(argv, k));
+                    vector_add(fun_argv, STRING_NEW(value));
+                }
+                
+                hash_put(gArrays, string_c_str(method->arg_name), fun_argv);
+            }
+
+            /// スタックフレームの記録を付ける
+            vector_add(gStackTraceFName, STRING_NEW(sname));
+            vector_add(gStackTraceLineNum, (void*)sline);
+
+            /// 実行 ///
+            *rcode = run(method->statments, title
+                , nextout, nextin, nexterr, return_, blocks, FALSE, envs, object2);
+            *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
+
+            /// スタックフレームの記録を消す
+            string_obj* sname_strace = vector_pop_back(gStackTraceFName);
+            int line_strace = (int)vector_pop_back(gStackTraceLineNum);
+
+
+            if(strcmp(string_c_str(method->arg_name), "") == 0) {
+                /// スタックを消す ///
+                hash_obj* stack = vector_pop_back(gStackFrame);
+
+                /// 引数を消す ///
+                hash_erase(gArrays, "ARGV");
+
+                /// 前の引数を元に戻す ///
+                if(fun_argv_before) {
+                    hash_put(gArrays, "ARGV", fun_argv_before);
+                }
+            }
+            else {
+                /// 引数を消す ///
+                hash_erase(gArrays, string_c_str(method->arg_name));
+
+                /// 前の引数を元に戻す ///
+                if(fun_argv_before) {
+                    hash_put(gArrays, string_c_str(method->arg_name)
+                                , fun_argv_before);
+                }
+            }
+
+            /// 終了コード ///
+            if(statment->mRCodeReverse) {
+                if(*rcode == 0) 
+                    *rcode = 1;
+                else
+                    *rcode = 0;
+            }
+
+            char buf2[256];
+            snprintf(buf2, 256, "%d", *rcode);
+            saphire_set_local_var("RCODE", buf2);
+
+            if(*rcode == -1) {
+                string_push_back(gErrMsg, string_c_str(sname_strace));
+                char tmp[1024];
+                snprintf(tmp, 1024, ":%d\n", line_strace);
+                string_push_back(gErrMsg, tmp);
+                
+                return FALSE;
+            }
+        }
+
         /// ユーザーコマンド ///
-        if(fun) {
+        else if(fun) {
             if(!statment_tree_redirect(command, &nextin, &nextout
                             , &nexterr, pipein, pipeout, pipeerr, sname, sline))
             {
@@ -746,7 +1047,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
             /// 実行 ///
             *rcode = run(fun->statments, title
-                , nextout, nextin, nexterr, return_, blocks, FALSE);
+                , nextout, nextin, nexterr, return_, blocks, FALSE, envs, object);
             *return_ = FALSE; // ユーザー関数が復帰したらreturn終わり
 
             /// スタックフレームの記録を消す
@@ -861,7 +1162,7 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
             }
 
             BOOL r = statment_tree_internal_commands(command, rcode
-                , argv, blocks, parent_blocks, nextout, nextin, nexterr, j, title, !first_program || exist_globalin || exist_stdinpipe,return_, sname, sline);
+                , argv, blocks, parent_blocks, nextout, nextin, nexterr, j, title, !first_program || exist_globalin || exist_stdinpipe,return_, sname, sline, envs, object);
 
             /// 終了コード ///
             if(statment->mRCodeReverse) {
@@ -905,13 +1206,13 @@ static BOOL statment_tree(sStatment* statment, int* last_program_pid, sWFd* pipe
 
                 nextin->mFd = -1;
 
-                if(!statment_tree_external_command(statment, argv, nextin, nextout, nexterr, pipeout, command, last_program, last_program_pid, sname, sline))
+                if(!statment_tree_external_command(statment, argv, nextin, nextout, nexterr, pipeout, command, last_program, last_program_pid, sname, sline, envs))
                 {
                     return FALSE;
                 }
             }
             else {
-                if(!statment_tree_external_command(statment, argv, nextin, nextout, nexterr, pipeout, command, last_program, last_program_pid,  sname, sline))
+                if(!statment_tree_external_command(statment, argv, nextin, nextout, nexterr, pipeout, command, last_program, last_program_pid,  sname, sline, envs))
                 {
                     return FALSE;
                 }
@@ -1094,7 +1395,7 @@ void run_wait_cprogs(sStatment* statment, int* rcode, sWFd* pipeout, int last_pr
 }
 
 // 複文を実行したい関数。本体。
-int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, vector_obj* parent_blocks, BOOL try_)
+int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr, BOOL* return_, vector_obj* parent_blocks, BOOL try_, vector_obj* envs, sObject* object)
 {
     int rcode = 0;
 
@@ -1108,13 +1409,13 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
         /// $によるコマンド展開
         if(statment->mNotEvaled) {
             string_obj* cmdline = STRING_NEW("");
-            if(expand_env(string_c_str(statment->mNotEvaled), cmdline, pipein, sname, &sline)) {
+            if(expand_env(string_c_str(statment->mNotEvaled), cmdline, pipein, sname, &sline, object)) {
                 sStatments* estatments = STATMENTS_NEW();
 
                 if(parse(string_c_str(cmdline), sname, &sline, estatments)) 
                 {
                     rcode = run(estatments, title, pipeout, pipein
-                            , pipeerr, return_, parent_blocks, FALSE);
+                        , pipeerr, return_, parent_blocks, FALSE, envs, object);
                     if(*return_) {
                         return 0;
                     }
@@ -1135,13 +1436,14 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
         else {
             sigchld_block(1);
 
+            vector_obj* psubs = VECTOR_NEW(30);
             int last_program_pid = -1;
-            int r = statment_tree(statment
+            BOOL r = statment_tree(statment
                           , &last_program_pid
                           , pipeout, pipein, pipeerr
                           , title, &rcode
-                          , return_, parent_blocks , sname, sline);
-
+                          , return_, parent_blocks , sname, sline
+                          , envs, psubs, object);
 
             /// プロセス回収 ///
             if(last_program_pid != -1) {
@@ -1156,6 +1458,28 @@ int run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pip
             if(*return_) {
                 return 0;
             }
+
+            /// プロセス置換を実行する ///
+            int l;
+            for(l=0; l<vector_size(psubs); l++) {
+                sPSub* psub = vector_item(psubs, l);
+
+                int f = open(string_c_str(psub->mFName), O_RDONLY);
+
+                sRFd* nextin2 = sRFd_new(f);
+                int rcode = run(psub->mStatments, "process substitution2"
+                    , pipeout, nextin2, 2, return_, parent_blocks, FALSE
+                    , envs, object);
+
+                (void)close(f);
+
+                if(rcode < 0) {
+                    return rcode;
+                }
+                if(*return_) {
+                    return 0;
+                }
+            }
         }
 
         /// 行末の処理 ///
@@ -1237,8 +1561,8 @@ int saphire_load(char* fname, sWFd* pipeout, sRFd* pipein, int pipeerr)
     (void)close(f);
 
     return saphire_shell(string_c_str(str), fname, pipeout, pipein, pipeerr);
-}
 
+}
 
 ///////////////////////////////////////////////////
 // シェル実行
@@ -1272,9 +1596,10 @@ int saphire_shell(char* command, char* fname, sWFd* pipeout, sRFd* pipein, int p
     else {
         /// 実行 ///
         int return_ = FALSE;
+        vector_obj* envs = VECTOR_NEW(10);
         int rcode = run(statments, string_c_str(fname2)
                     , pipeout, pipein, pipeerr
-                    , &return_, NULL, FALSE);
+                    , &return_, NULL, FALSE, envs, NULL);
 
         /// 解放 ///
         return rcode;
@@ -1300,6 +1625,63 @@ int saphire_shell3(string_obj* result, char* command, char* title, sRFd* pipein)
     return rcode;
 }
 
+// -1ならエラーメッセージがgErrMsgに入っているので出力してください
+int saphire_shell4(char* command, char* fname, sWFd* pipeout, sRFd* pipein, int pipeerr, sObject* object)
+{
+    gKitutukiSigInt = FALSE;
+
+    string_obj* fname2;
+    if(fname == NULL) {
+        fname2 = STRING_NEW(command);
+    }
+    else {
+        fname2 = STRING_NEW(fname);
+    }
+
+    int sline = 1;
+    string_obj* str = STRING_NEW("");
+    if(!delete_comments(command, str, " ", string_c_str(fname2), &sline)) {
+        return -1;
+    }
+
+    sline = 1;
+
+    sStatments* statments = STATMENTS_NEW();
+    if(!parse(string_c_str(str), string_c_str(fname2), &sline, statments)) {
+        return -1;
+    }
+    else {
+        /// 実行 ///
+        int return_ = FALSE;
+        vector_obj* envs = VECTOR_NEW(10);
+        int rcode = run(statments, string_c_str(fname2)
+                    , pipeout, pipein, pipeerr
+                    , &return_, NULL, FALSE, envs, object);
+
+        /// 解放 ///
+        return rcode;
+    }
+}
+
+// コマンド（文字列）を実行して結果をresultに入れたい関数
+// -1ならエラーメッセージがgErrMsgに入っているので出力してください
+int saphire_shell5(string_obj* result, char* command, char* title, sRFd* pipein, sObject* object)
+{
+    sWFd* pipeout = sWFd_new(-1);
+    int rcode = saphire_shell4(command, title, pipeout, pipein, STDERR_FILENO, object);
+    if(!sWFd_flash(pipeout)) {
+        return -1;
+    }
+
+    if(rcode < 0) {
+        return -1;
+    }
+
+    string_put(result, pipeout->mBuffer);
+
+    return rcode;
+}
+
 ///////////////////////////////////////////////////
 // ジョブの数
 ///////////////////////////////////////////////////
@@ -1510,8 +1892,9 @@ BOOL saphire_compile2(char* cmdline, char* fname, sStatments* statments)
 int saphire_run(sStatments* statments, char* title, sWFd* pipeout, sRFd* pipein, int pipeerr)
 {
     int return_ = FALSE;
+    vector_obj* envs = VECTOR_NEW(10);
     int r = run(statments, title, pipeout, pipein, pipeerr
-            , &return_, NULL, FALSE);
+            , &return_, NULL, FALSE, envs, NULL);
 
     return r;
 }
@@ -1526,12 +1909,12 @@ void tmpfile_sweep()
 {
     /// プロセス置換で使った一時ファイルを掃除 ///
     int k;
-    for(k=0; k<vector_size(gPSubTmpFiles); k++) {
-        string_obj* fname = vector_item(gPSubTmpFiles, k);
+    for(k=0; k<vector_size(gTmpFiles); k++) {
+        string_obj* fname = vector_item(gTmpFiles, k);
 
         (void)unlink(string_c_str(fname));
     }
-    vector_clear(gPSubTmpFiles);
+    vector_clear(gTmpFiles);
 }
 
 /// ごみ掃除(GCはないけど)したい関数
